/*! \file mpii.h
 * \brief Header to use insted of mpi.h
 *
 * This header, when used instead of (or after) mpi.h will override the functions
 * we replace with their auto-optimized variants.
 *
 * This is only necesarry for static injection.
 */

#ifndef _INJ_MPII_H
#define _INJ_MPII_H

#include <mpi.h>

/* Make sure the include works in both C and C++ programs! */

#if defined(__cplusplus)
extern "C" {
#endif

/* Prototypes for our injected versions, identical to the originals. */

int Inj_MPI_Init(int *, char ***);
int Inj_MPI_Send(void *, int, MPI_Datatype, int, int, MPI_Comm);
int Inj_MPI_Recv(void *, int, MPI_Datatype, int, int, MPI_Comm, MPI_Status *);
int Inj_MPI_Sendrecv(void *, int, MPI_Datatype, int, int, void *, int, MPI_Datatype, int, int, MPI_Comm, MPI_Status *);
int Inj_MPI_Get_count(MPI_Status *, MPI_Datatype, int *);

void *Inj_malloc(size_t size);
void Inj_free(void *ptr);

#if defined(__cplusplus)
}
#endif

/* Use preprocessor trick to use our version instead of the original. */

#define MPI_Init Inj_MPI_Init
#define MPI_Send Inj_MPI_Send
#define MPI_Recv Inj_MPI_Recv
#define MPI_Sendrecv Inj_MPI_Sendrecv
#define MPI_Get_count Inj_MPI_Get_count

#define malloc Inj_malloc
#define free Inj_free

#endif
