/*! \file stattest.cxx
 * \brief Test MPI_Get_count
 *
 * This tests the chaining of MPI_Get_count in a simple
 * one-to-one transfer.
 */
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <mpi.h>

// Static injection test
#include "mpii.h"

int rank;
int nump;

main(int argc, char **argv )
{
  int n = 8;

  MPI_Init(&argc, &argv);
  
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &nump);

  if (nump != 2) {
	fprintf(stderr, "nump != 2\n");
	exit(-1);
  }

  unsigned char *buffer=(unsigned char *) malloc(128);
  unsigned char *rbuffer=(unsigned char *) malloc(128);
  for(int i=0;i<128;i++)
	buffer[i]=rank*128+i;

  fprintf(stderr,"Rank %d:Alloc %p %p\n", rank,buffer,rbuffer);

  int other_rank = (rank + 1) % 2;

  MPI_Status status;
  MPI_Sendrecv(buffer, 128, MPI_UNSIGNED_CHAR, other_rank, 0, rbuffer, 128, MPI_UNSIGNED_CHAR, other_rank, 0, MPI_COMM_WORLD, &status);
  fprintf(stderr,"Rank %d: Oooout\n",rank);

  int size;
  MPI_Get_count(&status, MPI_UNSIGNED_CHAR, &size);
  fprintf(stderr,"Rank %d: Size %d\n", rank, size);

  int sum = 0;
  int valid = 0;
  for(int i=0;i<128;i++) {
	sum += buffer[i];
	sum += rbuffer[i];
	valid += i;
	valid += (i+128);
  }
  fprintf(stderr,"Rank %d: Sum %d Valid %d\n", rank, sum, valid);

  MPI_Finalize();
}
